--- old/usr/src/cmd/sgs/libld/common/place.c	2013-01-02 21:49:16.327886309 -0500
+++ new/usr/src/cmd/sgs/libld/common/place.c	2013-01-02 21:49:16.339101289 -0500
@@ -851,6 +851,13 @@
 	}
 
 	/*
+	 * When building relocatable objects, we must not redirect COMDAT
+	 * section names into their outputs, such that our output object may
+	 * be successfully used as an input object also requiring COMDAT
+	 * processing
+	 */
+
+	/*
 	 * GNU section names may follow the convention:
 	 *
 	 *	.gnu.linkonce.*
@@ -863,13 +870,15 @@
 	 * because we know the name is not NULL, and therefore must have
 	 * at least one character plus a NULL termination.
 	 */
-	if (((ofl->ofl_flags & FLG_OF_RELOBJ) == 0) &&
-	    (isp->is_name == oname) && (isp->is_name[1] == 'g') &&
+	if ((isp->is_name == oname) && (isp->is_name[1] == 'g') &&
 	    (strncmp(MSG_ORIG(MSG_SCN_GNU_LINKONCE), isp->is_name,
 	    MSG_SCN_GNU_LINKONCE_SIZE) == 0)) {
-		if ((oname =
-		    (char *)gnu_linkonce_sec(isp->is_name)) != isp->is_name) {
-			DBG_CALL(Dbg_sec_redirected(ofl->ofl_lml, isp, oname));
+		if ((ofl->ofl_flags & FLG_OF_RELOBJ) == 0) {
+			if ((oname = (char *)gnu_linkonce_sec(isp->is_name)) !=
+			    isp->is_name) {
+				DBG_CALL(Dbg_sec_redirected(ofl->ofl_lml, isp,
+				    oname));
+			}
 		}
 
 		/*
@@ -894,16 +903,17 @@
 	 * sections, and this identification can be triggered by a pattern
 	 * match section names.
 	 */
-	if (((ofl->ofl_flags & FLG_OF_RELOBJ) == 0) &&
-	    (isp->is_name == oname) && (isp->is_flags & FLG_IS_COMDAT) &&
+	if ((isp->is_name == oname) && (isp->is_flags & FLG_IS_COMDAT) &&
 	    ((sname = gnu_comdat_sym(ifl, isp)) != NULL)) {
 		size_t	size = sname - isp->is_name;
 
-		if ((oname = libld_malloc(size + 1)) == NULL)
-			return ((Os_desc *)S_ERROR);
-		(void) strncpy(oname, isp->is_name, size);
-		oname[size] = '\0';
-		DBG_CALL(Dbg_sec_redirected(ofl->ofl_lml, isp, oname));
+		if ((ofl->ofl_flags & FLG_OF_RELOBJ) == 0) {
+			if ((oname = libld_malloc(size + 1)) == NULL)
+				return ((Os_desc *)S_ERROR);
+			(void) strncpy(oname, isp->is_name, size);
+			oname[size] = '\0';
+			DBG_CALL(Dbg_sec_redirected(ofl->ofl_lml, isp, oname));
+		}
 
 		/*
 		 * Enable relaxed relocation processing, as this is
@@ -1199,6 +1209,7 @@
 	 * so that they can be updated as a group later.
 	 */
 	if ((shdr->sh_type == SHT_GROUP) &&
+	    ((isp->is_flags & FLG_IS_DISCARD) == 0) &&
 	    (aplist_append(&ofl->ofl_osgroups, osp,
 	    AL_CNT_OFL_OSGROUPS) == NULL))
 		return ((Os_desc *)S_ERROR);
